#!/usr/bin/perl
#
# [KSC4-PERL]---------------------------------------------------------------[]
#
#  CODESIZE.PL - Gathers resource usage data for different compilation options
#                Version 0.1 by Marco Carnut <kiko at postcogito dot org>
#
# []------------------------------------------------------------------------[]

#my $fixed_opts = "-DQUELL_BUILTIN_CONFIG";
#my $fixed_opts = "-DQUELL_BUILTIN_CONFIG -DHAVE_CONSTANTS_ON_EEPROM";
#my $fixed_opts = "-DQUELL_BUILTIN_CONFIG -DUSE_REGISTER_VARS";
my $fixed_opts = "-DQUELL_BUILTIN_CONFIG -DUSE_REGISTER_VARS -DHAVE_CONSTANTS_ON_EEPROM";

my %vary_opts = (
    "DO_BLINK_COLON" => "", 
    "HAVE_INSTRUMENTATION" => "",
    "HAVE_UART" => "",
    "HAVE_ESCAPE_TO_BOOTLOADER" => "HAVE_UART",
    "HAVE_MUX_DEMO" => ""
);

my %abbrevs = (
    "DO_BLINK_COLON" => "COLON", 
    "HAVE_INSTRUMENTATION" => "INSTR",
    "HAVE_UART" => "UART",
    "HAVE_ESCAPE_TO_BOOTLOADER" => "E_BOOT",
    "HAVE_MUX_DEMO" => "MUXD"
);

my @vops=sort keys %vary_opts;
my $total=2**(1+$#vops);
my %history;

print "NUM FLASH RAM EEP ";
print num_to_string($setnum,2),"\n";
my $item;
for $count (0..$total-1) {
    if (!$history{$count}) {
        printf("%3d ",++$item);
        $setnum=fix_deps($count);
        $history{$setnum}=$count;
        $ENV{'DEFINES'}=$fixed_opts . " " . num_to_string($setnum,1);
        $ENV{'CODELIMIT'}=100000;
        my $text=`make -B ktimer.hex`;
        if ($?) {
            print "error on $ENV{DEFINES}\n";
            exit 1;
        } else {
            my ($flash)  = $text =~  m/ FLASH: (\d+) bytes/o;
            my ($ram)    = $text =~  m/   RAM: (\d+) bytes/o;
            my ($eeprom) = $text =~  m/EEPROM: (\d+) bytes/o;
            printf("%5d %3d %3d ",$flash,$ram,$eeprom);
            print num_to_string($setnum,3),"\n";
            #exit if $count>2;
        }
    } #else {
    #    print "$count => $history{$count}\n";
    #}
}

sub fix_deps
{
    my ($setnum) = @_;
    my $bit=1;
    my %val;
    for $o (@vops) {
        $val{$o}=$bit;
        $bit<<=1;
    }
    $bit=1;
    my $out=0;
    for $o (@vops) {
        if ($setnum & $bit) {
            $out |= $bit;
            for $oo (split(/:/o,$vary_opts{$o})) {
                $out |= $val{$oo};
            }
        }
        $bit<<=1;
    }
    return $out;
}

sub num_to_string
{
    my ($setnum,$mode) = @_;
    my $bit=1;
    my $out="";
    my $width=7;
    for $o (@vops) {
        if ($mode == 2) {
            my $text = $abbrevs{$o};
            my $len  = length($text);
            my $sl   = int(($width-$len+(len%2?0:1))/2);
            my $sr   = ($width+1)-$sl-$len;
            my $text =(" " x $sl) . $text . (" " x $sr);
            #my $text =$sl . $text .  $sr . "|";
            $out .= $text;
        }
        if ($setnum & $bit) { 
            $out .= "-D$o "                          if $mode == 1;
            $out .= sprintf("   1    ")             if $mode == 3;
        } else {
            $out .= sprintf("   0    ")             if $mode == 3;
        }
        $bit<<=1;
    }
    return $out;
}
